/*
 * Copyright (C) 2017 - 2018 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <xdc/std.h>
#include <xdc/runtime/Error.h>
#include <xdc/runtime/System.h>
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>

#include <ti/csl/soc.h>
#include <ti/csl/cslr_device.h>
#if defined (SOC_AM574x)
#include <ti/csl/soc/am574x/src/csl_device_xbar.h>
#else
#include <ti/csl/soc/am572x/src/csl_device_xbar.h>
#endif

#include <ti/ndk/inc/netmain.h>
#include <ti/ndk/inc/stkmain.h>
#include "ti/ndk/inc/os/osif.h"

#include <ti/board/board.h>

#include <ti/drv/uart/UART.h>
#include <ti/drv/uart/UART_stdio.h>

#include <ti/drv/emac/emac_drv.h>
#include <ti/drv/emac/src/v4/emac_drv_v4.h>

#ifdef NIMU_FTP_APP
#include <ti/transport/ndk/nimu/example/ftpApp/ftpserver/ftpserver.h>
#endif

#include <limits.h>

#include <xdc/runtime/Types.h>
#include <xdc/runtime/Timestamp.h>
#include <ti/csl/csl_tsc.h>
#include <ti/sysbios/hal/Hwi.h>

#include "tcptest_config.h"
#include "tcptest_osal.h"

extern char *LocalIPAddr;
extern void app_stats(UArg arg0, UArg arg1);
#ifdef NIMU_DUAL_MAC_MODE
    /* Static IP Address settings for interface 1 */
    char *ipAddr      = "192.168.2.4";
    char *ipMask      = "255.255.255.0";
    char *ipGateway   = "192.168.2.1";
#endif

/* ========================================================================== */
/*                             Macros                                  */
/* ========================================================================== */
/* Enable the below macro to have prints on the IO Console */

//#define IO_CONSOLE

#ifndef IO_CONSOLE
#define NIMU_log                TCPTEST_printf
#else
#define NIMU_log                printf
#endif

#define MAX_TABLE_ENTRIES   3
#define LISTEN_TIMEOUT      (10000)

/* ========================================================================== */
/*                            Global Variables                                */
/* ========================================================================== */

static int nimu_device_index = 0U;

NIMU_DEVICE_TABLE_ENTRY NIMUDeviceTable[MAX_TABLE_ENTRIES];

static void *pvTcpConnectionHandler = NULL;


extern int CpswEmacInit (STKEVENT_Handle hEvent);
extern int iINetPollingLoop(SOCKET xServerSocket, uint32_t xFlags);

int iDaemonFxn(SOCKET xServerSocket, uint32_t xArguments);

void vRtosINetOpenHook();
void vRtosINetCloseHook();
void vRtosINetIPAddrHook(int iIPAddr, int iIfIdx, int iAdded);
void vRtosINetServiceReportHook(char Item, char Status,char Report, void * h);
void vRtosINetLinkHook(int iLinkState);

int main()
{
    /* Call board init functions */
    Board_initCfg boardCfg;

    boardCfg =  BOARD_INIT_PINMUX_CONFIG |
        BOARD_INIT_MODULE_CLOCK | BOARD_INIT_UART_STDIO;
#if defined (SOC_AM574x) && defined(NIMU_DUAL_MAC_MODE)
    boardCfg |= BOARD_INIT_ETH_PHY;
#endif
    Board_init(boardCfg);

#ifdef _TMS320C6X
    CSL_xbarDspIrqConfigure(1,CSL_XBAR_INST_DSP1_IRQ_75, CSL_XBAR_GMAC_SW_IRQ_RX_PULSE);
    CSL_xbarDspIrqConfigure(1,CSL_XBAR_INST_DSP1_IRQ_76, CSL_XBAR_GMAC_SW_IRQ_TX_PULSE);
#elif defined (__TI_ARM_V7M4__)
    CSL_xbarIpuIrqConfigure(1, CSL_XBAR_INST_IPU1_IRQ_60, CSL_XBAR_GMAC_SW_IRQ_RX_PULSE);
    CSL_xbarIpuIrqConfigure(1, CSL_XBAR_INST_IPU1_IRQ_61, CSL_XBAR_GMAC_SW_IRQ_TX_PULSE);
#elif defined(__ARM_ARCH_7A__)
    CSL_xbarMpuIrqConfigure(CSL_XBAR_INST_MPU_IRQ_92, CSL_XBAR_GMAC_SW_IRQ_RX_PULSE);
    CSL_xbarMpuIrqConfigure(CSL_XBAR_INST_MPU_IRQ_93, CSL_XBAR_GMAC_SW_IRQ_TX_PULSE);
#endif
    /* Select RGMII 2 ports GMIIx_SEL = 2 for RGMII*/
    CSL_FINS (((CSL_control_coreRegs *) CSL_MPU_CTRL_MODULE_CORE_CORE_REGISTERS_REGS)->CONTROL_IO_1,
          CONTROL_CORE_CONTROL_IO_1_GMII1_SEL, 2U);
    CSL_FINS (((CSL_control_coreRegs *) CSL_MPU_CTRL_MODULE_CORE_CORE_REGISTERS_REGS)->CONTROL_IO_1,
          CONTROL_CORE_CONTROL_IO_1_GMII2_SEL, 2U);

    /*GMAC RESET ISOLATION Enable*/
    CSL_FINS (((CSL_control_coreRegs *) CSL_MPU_CTRL_MODULE_CORE_CORE_REGISTERS_REGS)->CONTROL_IO_2,
          CONTROL_CORE_CONTROL_IO_2_GMAC_RESET_ISOLATION_ENABLE, 0U);
    CSL_FINS (((CSL_control_coreRegs *) CSL_MPU_CTRL_MODULE_CORE_CORE_REGISTERS_REGS)->CONTROL_IO_2,
          CONTROL_CORE_CONTROL_IO_2_GMAC_RESET_ISOLATION_ENABLE, 1U);

#ifdef evmAM572x
    EMAC_HwAttrs_V4 cfg;
    EMAC_socGetInitCfg(0, &cfg);
    cfg.port[0].phy_addr = GMAC_PORT1_ETHERNET_PHY_ADRESS;
    cfg.port[1].phy_addr = GMAC_PORT2_ETHERNET_PHY_ADRESS;
    EMAC_socSetInitCfg(0, &cfg);
#endif

    NIMUDeviceTable[nimu_device_index++].init =  &CpswEmacInit;

#ifdef NIMU_DUAL_MAC_MODE
    NIMUDeviceTable[nimu_device_index++].init =  &CpswEmacInit;
#endif
    NIMUDeviceTable[nimu_device_index].init =  NULL;

    BIOS_start();

    return -1;
}

int iDaemonFxn(SOCKET xServerSocket, uint32_t xArguments)
{
    static const char fn[] = "DmonFxn";

    TCPTEST_printf("%s: INet client connected, socket=%x\n", fn, xServerSocket);

    fdShare(xServerSocket);

    int iRetVal = iINetPollingLoop(xServerSocket, TCPTEST_RECV_FLAGS);

    return iRetVal;
}

void vRtosINetOpenHook()
{
    static const char fn[] = "INetOpH";

    TCPTEST_printf("%s: INet network open hook called\n", fn);

    pvTcpConnectionHandler = DaemonNew(SOCK_STREAMNC,
                                       0,
                                       TCPTEST_TCP_PORT,
                                       iDaemonFxn,
                                       1,
                                       0x8000,
                                       0,
                                       3);

    if (pvTcpConnectionHandler == 0 /* NULL */)
    {
        TCPTEST_printf("%s: Error creating INet connection task\n", fn);
        pvTcpConnectionHandler = NULL;
    }
}

void vRtosINetCloseHook()
{
    static const char fn[] = "INetClH";

    TCPTEST_printf("%s: INet network close hook called\n", fn);

    if ( pvTcpConnectionHandler != NULL )
    {
        DaemonFree( pvTcpConnectionHandler );
        pvTcpConnectionHandler = NULL;
    }
}

void vRtosINetIPAddrHook(int iIPAddr, int iIfIdx, int iAdded)
{
    static const char fn[] = "INetIpH";

    uint32_t IPTmp;

    TCPTEST_printf("%s: INet network IP address binding hook called\n", fn);

    if (iAdded)
        UART_printf("Network Added: ");
    else
        UART_printf("Network Removed: ");

    // Print a message
    IPTmp = NDK_ntohl(iIPAddr);
    TCPTEST_printf("If-%d: %d.%d.%d.%d:%d\n", iIfIdx, (uint8_t) (IPTmp >> 24) & 0xFF,
                   (uint8_t) (IPTmp >> 16) & 0xFF, (uint8_t) (IPTmp >> 8) & 0xFF,
                   (uint8_t) IPTmp & 0xFF, TCPTEST_TCP_PORT);

    return;
}

void vRtosINetServiceReportHook(char Item, char Status,char Report, void * h)
{
    static const char fn[] = "INetSrR";

    // Service Status Reports
    static const char *TaskName[] = { "Telnet","HTTP","NAT","DHCPS","DHCPC","DNS" };
    static const char *ReportStr[] = { "","Running","Updated","Complete","Fault" };
    static const char *StatusStr[] = { "Disabled", "Waiting", "IPTerm", "Failed", "Enabled" };

    TCPTEST_printf("%s: INet network service report hook called\n", fn);

    TCPTEST_printf("%s: Service Status: %-9s: %-9s: %-9s: %03d\n", fn,
         TaskName[(int)Item-1], StatusStr[(int)Status], ReportStr[(int)Report/256], Report&0xFF );

    return;
}

void vRtosINetLinkHook(int iLinkState)
{
    static const char fn[] = "INetLnk";

    TCPTEST_printf("%s: INet network link hook called\n", fn);

    if (iLinkState == 0)
    {
        TCPTEST_printf("%s: INet network if down\n", fn);
    }
    else if (iLinkState == 1)
    {
        TCPTEST_printf("%s: INet network if up\n", fn);
    }
    else
    {
        TCPTEST_printf("%s: Unknown network state received, state=%d\n",
             fn, iLinkState);
    }

    return;
}
