#include "tcptest_config.h"
#include "tcptest_osal.h"

static int  iBytesReadTotal = 0;

int iINetReceiver(SOCKET xServerSocket, int iFlags)
{
    static const char fn[] = "INetRcv";

    int iBytesRead = 0;
    char aucRecvBuff[1024];

    do
    {
        iBytesRead = TCPTEST_recv(xServerSocket, aucRecvBuff, sizeof(aucRecvBuff), iFlags);
        if (iBytesRead > 0)
        {
            iBytesReadTotal += iBytesRead;

            if (iBytesReadTotal > TCPTEST_BYTES_READ_TRESHOLD)
            {
                TCPTEST_printf("%s: bytes_read=%d\n", fn, iBytesReadTotal);

                iBytesReadTotal = 0;
            }
        }
        else if (SOCKET_ERROR == iBytesRead)
        {
            int iRecvError = TCPTEST_error();

            if (iRecvError == TCPTEST_EAGAIN)
            {
                // No data to read. If non-blocking socket (flag RTOS_MSG_DONTWAIT)
                // and no data is available, then instead of blocking recv()
                // returns immediately with the error RTOS_EAGAIN (a.k.a.
                // RTOS_EWOULDBLOCK)
                TCPTEST_printf("%s: No data to read from socket\n", fn);
                break;
            }
            else
            {
                // Connection error
                TCPTEST_printf("%s: Error while receiving data from a socket, retval=%d, error=%d, bytes_read=%d\n",
                               fn, iBytesRead, iRecvError, iBytesReadTotal);
                return (0);
            }
        }
        else
        {
            // Connection closed, no more data to read
            TCPTEST_printf("%s: Connection closed by the peer, bytes_read=%d\n",
                           fn, iBytesReadTotal);
            return (0);
        }
    }
    while (iBytesRead > 0);

    if (iBytesReadTotal > 0)
    {
        TCPTEST_printf("%s: Finished reading data from socket, bytes_read=%d\n",
                       fn, iBytesReadTotal);
    }

    return (1);
}


int iINetPollingLoop(SOCKET xServerSocket, uint32_t xFlags)
{
    static const char fn[] = "INetLop";

    TCPTEST_printf("%s: Start endless loop to receive data from socket=%x\n", fn, xServerSocket);

    iBytesReadTotal = 0;

    int iDone = 0;
    while (iDone == 0)
    {
        if (iINetReceiver(xServerSocket, xFlags) == 0)
        {
            TCPTEST_printf("%s: Receiving data from socket=%x failed, stop endless loop\n",
                           fn, xServerSocket);
            iDone = 1;
        }

        TCPTEST_sleep(1000);
    }

    TCPTEST_printf("%s: Endless loop finished, socket=%x\n", fn, xServerSocket);

    return(1);
}
